from typing import overload
from enum import Enum
import datetime
import typing

import Common.Data.Consolidators
import QuantConnect
import QuantConnect.Data
import QuantConnect.Data.Consolidators
import QuantConnect.Data.Market
import QuantConnect.Securities


class DollarVolumeRenkoConsolidator(QuantConnect.Data.Consolidators.VolumeRenkoConsolidator):
    """
    This consolidator transforms a stream of BaseData instances into a stream of RenkoBar
    with a constant dollar volume for each bar.
    """

    def __init__(self, bar_size: float) -> None:
        """
        Initializes a new instance of the DollarVolumeRenkoConsolidator class using the specified .
        
        :param bar_size: The constant dollar volume size of each bar
        """
        ...

    def adjust_volume(self, volume: float, price: float) -> float:
        """
        Converts raw volume into dollar volume by multiplying it with the trade price.
        
        This method is protected.
        
        :param volume: The raw trade volume
        :param price: The trade price
        :returns: The dollar volume.
        """
        ...


class SessionConsolidator(QuantConnect.Data.Consolidators.PeriodCountConsolidatorBase[QuantConnect.Data.BaseData, QuantConnect.Data.Market.SessionBar]):
    """Consolidates intraday market data into a single daily SessionBar (OHLCV + OpenInterest)."""

    def __init__(self, exchange_hours: QuantConnect.Securities.SecurityExchangeHours, source_tick_type: QuantConnect.TickType, symbol: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract]) -> None:
        """
        Initializes a new instance of the SessionConsolidator class.
        
        :param exchange_hours: The exchange hours
        :param source_tick_type: Type of the source tick
        :param symbol: The symbol
        """
        ...

    def aggregate_bar(self, working_bar: QuantConnect.Data.Market.SessionBar, data: QuantConnect.Data.BaseData) -> None:
        """
        Aggregates the new 'data' into the 'working_bar'
        
        This method is protected.
        
        :param working_bar: The bar we're building, null if the event was just fired and we're starting a new trade bar
        :param data: The new data
        """
        ...

    def on_data_consolidated(self, e: QuantConnect.Data.Market.SessionBar) -> None:
        """
        Event handler that fires when a new piece of data is produced
        
        This method is protected.
        """
        ...

    def reset(self) -> None:
        """Resets the consolidator"""
        ...

    def reset_working_bar(self) -> None:
        """
        Resets the working bar
        
        This method is protected.
        """
        ...

    def validate_and_scan(self, current_local_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Validates the current local time and triggers Scan() if a new day is detected.
        
        :param current_local_time: The current local time.
        """
        ...





from typing import overload
from enum import Enum
import abc
import typing
import warnings

import System
import System.Collections
import System.Globalization
import System.Runtime.Serialization


class IEnumerator(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    @abc.abstractmethod
    def current(self) -> System.Object:
        ...

    def move_next(self) -> bool:
        ...

    def reset(self) -> None:
        ...


class IEnumerable(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def get_enumerator(self) -> System.Collections.IEnumerator:
        ...


class ICollection(System.Collections.IEnumerable, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    @abc.abstractmethod
    def count(self) -> int:
        ...

    @property
    @abc.abstractmethod
    def sync_root(self) -> System.Object:
        ...

    @property
    @abc.abstractmethod
    def is_synchronized(self) -> bool:
        ...

    def copy_to(self, array: System.Array, index: int) -> None:
        ...


class IDictionary(System.Collections.ICollection, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    @abc.abstractmethod
    def keys(self) -> System.Collections.ICollection:
        ...

    @property
    @abc.abstractmethod
    def values(self) -> System.Collections.ICollection:
        ...

    @property
    @abc.abstractmethod
    def is_read_only(self) -> bool:
        ...

    @property
    @abc.abstractmethod
    def is_fixed_size(self) -> bool:
        ...

    def __getitem__(self, key: typing.Any) -> typing.Any:
        ...

    def __setitem__(self, key: typing.Any, value: typing.Any) -> None:
        ...

    def add(self, key: typing.Any, value: typing.Any) -> None:
        ...

    def clear(self) -> None:
        ...

    def contains(self, key: typing.Any) -> bool:
        ...

    def remove(self, key: typing.Any) -> None:
        ...


class IHashCodeProvider(metaclass=abc.ABCMeta):
    """
    Provides a mechanism for a Hashtable user to override the default
    GetHashCode() function on Objects, providing their own hash function.
    
    IHashCodeProvider has been deprecated. Use IEqualityComparer instead.
    """

    def get_hash_code(self, obj: typing.Any) -> int:
        """Returns a hash code for the given object."""
        ...


class IComparer(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def compare(self, x: typing.Any, y: typing.Any) -> int:
        ...


class IEqualityComparer(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def equals(self, x: typing.Any, y: typing.Any) -> bool:
        ...

    def get_hash_code(self, obj: typing.Any) -> int:
        ...


class DictionaryEntry:
    """This class has no documentation."""

    @property
    def key(self) -> System.Object:
        ...

    @key.setter
    def key(self, value: System.Object) -> None:
        ...

    @property
    def value(self) -> System.Object:
        ...

    @value.setter
    def value(self, value: System.Object) -> None:
        ...

    def __init__(self, key: typing.Any, value: typing.Any) -> None:
        ...

    def deconstruct(self, key: typing.Optional[typing.Any], value: typing.Optional[typing.Any]) -> typing.Tuple[None, typing.Any, typing.Any]:
        ...

    def to_string(self) -> str:
        ...


class IDictionaryEnumerator(System.Collections.IEnumerator, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    @abc.abstractmethod
    def key(self) -> System.Object:
        ...

    @property
    @abc.abstractmethod
    def value(self) -> System.Object:
        ...

    @property
    @abc.abstractmethod
    def entry(self) -> System.Collections.DictionaryEntry:
        ...


class Hashtable(System.Object, System.Collections.IDictionary, System.Runtime.Serialization.ISerializable, System.Runtime.Serialization.IDeserializationCallback, System.ICloneable):
    """This class has no documentation."""

    @property
    def hcp(self) -> System.Collections.IHashCodeProvider:
        """
        This property is protected.
        
        Hashtable.hcp has been deprecated. Use the EqualityComparer property instead.
        """
        warnings.warn("Hashtable.hcp has been deprecated. Use the EqualityComparer property instead.", DeprecationWarning)

    @hcp.setter
    def hcp(self, value: System.Collections.IHashCodeProvider) -> None:
        warnings.warn("Hashtable.hcp has been deprecated. Use the EqualityComparer property instead.", DeprecationWarning)

    @property
    def comparer(self) -> System.Collections.IComparer:
        """
        This property is protected.
        
        Hashtable.comparer has been deprecated. Use the KeyComparer properties instead.
        """
        warnings.warn("Hashtable.comparer has been deprecated. Use the KeyComparer properties instead.", DeprecationWarning)

    @comparer.setter
    def comparer(self, value: System.Collections.IComparer) -> None:
        warnings.warn("Hashtable.comparer has been deprecated. Use the KeyComparer properties instead.", DeprecationWarning)

    @property
    def equality_comparer(self) -> System.Collections.IEqualityComparer:
        """This property is protected."""
        ...

    @property
    def is_read_only(self) -> bool:
        ...

    @property
    def is_fixed_size(self) -> bool:
        ...

    @property
    def is_synchronized(self) -> bool:
        ...

    @property
    def keys(self) -> System.Collections.ICollection:
        ...

    @property
    def values(self) -> System.Collections.ICollection:
        ...

    @property
    def sync_root(self) -> System.Object:
        ...

    @property
    def count(self) -> int:
        ...

    def __contains__(self, key: typing.Any) -> bool:
        ...

    def __getitem__(self, key: typing.Any) -> typing.Any:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, capacity: int) -> None:
        ...

    @overload
    def __init__(self, capacity: int, load_factor: float) -> None:
        ...

    @overload
    def __init__(self, capacity: int, load_factor: float, equality_comparer: System.Collections.IEqualityComparer) -> None:
        ...

    @overload
    def __init__(self, equality_comparer: System.Collections.IEqualityComparer) -> None:
        ...

    @overload
    def __init__(self, capacity: int, equality_comparer: System.Collections.IEqualityComparer) -> None:
        ...

    @overload
    def __init__(self, d: System.Collections.IDictionary) -> None:
        ...

    @overload
    def __init__(self, d: System.Collections.IDictionary, load_factor: float) -> None:
        ...

    @overload
    def __init__(self, d: System.Collections.IDictionary, equality_comparer: System.Collections.IEqualityComparer) -> None:
        ...

    @overload
    def __init__(self, d: System.Collections.IDictionary, load_factor: float, equality_comparer: System.Collections.IEqualityComparer) -> None:
        ...

    @overload
    def __init__(self, hcp: System.Collections.IHashCodeProvider, comparer: System.Collections.IComparer) -> None:
        """This constructor has been deprecated. Use Hashtable(IEqualityComparer) instead."""
        ...

    @overload
    def __init__(self, capacity: int, hcp: System.Collections.IHashCodeProvider, comparer: System.Collections.IComparer) -> None:
        """This constructor has been deprecated. Use Hashtable(int, IEqualityComparer) instead."""
        ...

    @overload
    def __init__(self, d: System.Collections.IDictionary, hcp: System.Collections.IHashCodeProvider, comparer: System.Collections.IComparer) -> None:
        """This constructor has been deprecated. Use Hashtable(IDictionary, IEqualityComparer) instead."""
        ...

    @overload
    def __init__(self, capacity: int, load_factor: float, hcp: System.Collections.IHashCodeProvider, comparer: System.Collections.IComparer) -> None:
        """This constructor has been deprecated. Use Hashtable(int, float, IEqualityComparer) instead."""
        ...

    @overload
    def __init__(self, d: System.Collections.IDictionary, load_factor: float, hcp: System.Collections.IHashCodeProvider, comparer: System.Collections.IComparer) -> None:
        """This constructor has been deprecated. Use Hashtable(IDictionary, float, IEqualityComparer) instead."""
        ...

    @overload
    def __init__(self, info: System.Runtime.Serialization.SerializationInfo, context: System.Runtime.Serialization.StreamingContext) -> None:
        """
        This method is protected.
        
        Obsoletions.LegacyFormatterImplMessage
        """
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, key: typing.Any, value: typing.Any) -> None:
        ...

    def add(self, key: typing.Any, value: typing.Any) -> None:
        ...

    def clear(self) -> None:
        ...

    def clone(self) -> System.Object:
        ...

    def contains(self, key: typing.Any) -> bool:
        ...

    def contains_key(self, key: typing.Any) -> bool:
        ...

    def contains_value(self, value: typing.Any) -> bool:
        ...

    def copy_to(self, array: System.Array, array_index: int) -> None:
        ...

    def get_enumerator(self) -> System.Collections.IDictionaryEnumerator:
        ...

    def get_hash(self, key: typing.Any) -> int:
        """This method is protected."""
        ...

    def get_object_data(self, info: System.Runtime.Serialization.SerializationInfo, context: System.Runtime.Serialization.StreamingContext) -> None:
        """Obsoletions.LegacyFormatterImplMessage"""
        warnings.warn("Obsoletions.LegacyFormatterImplMessage", DeprecationWarning)

    def key_equals(self, item: typing.Any, key: typing.Any) -> bool:
        """This method is protected."""
        ...

    def on_deserialization(self, sender: typing.Any) -> None:
        ...

    def remove(self, key: typing.Any) -> None:
        ...

    @staticmethod
    def synchronized(table: System.Collections.Hashtable) -> System.Collections.Hashtable:
        ...


class BitArray(System.Object, System.Collections.ICollection, System.ICloneable, System.Runtime.Serialization.ISerializable):
    """
    Manages a compact array of bit values, which are represented as bool, where
    true indicates that the bit is on (1) and false indicates
    the bit is off (0).
    """

    @property
    def length(self) -> int:
        """Gets or sets the number of elements in the BitArray."""
        ...

    @length.setter
    def length(self, value: int) -> None:
        ...

    @property
    def count(self) -> int:
        """Gets the number of elements contained in the BitArray."""
        ...

    @property
    def sync_root(self) -> System.Object:
        """Gets an object that can be used to synchronize access to the BitArray."""
        ...

    @property
    def is_synchronized(self) -> bool:
        """Gets a value indicating whether access to the BitArray is synchronized (thread safe)."""
        ...

    @property
    def is_read_only(self) -> bool:
        """Gets a value indicating whether the BitArray is read-only."""
        ...

    def __getitem__(self, index: int) -> bool:
        """
        Gets or sets the value of the bit at a specific position in the BitArray.
        
        :param index: The zero-based index of the value to get or set.
        :returns: The value of the bit at position .
        """
        ...

    @overload
    def __init__(self, length: int) -> None:
        """
        Initializes a new instance of the BitArray class that can hold the specified
        number of bit values, which are initially set to false.
        
        :param length: The number of bit values in the new BitArray.
        """
        ...

    @overload
    def __init__(self, length: int, default_value: bool) -> None:
        """
        Initializes a new instance of the BitArray class that can hold the specified number of
        bit values, which are initially set to the specified value.
        
        :param length: The number of bit values in the new BitArray.
        :param default_value: The Boolean value to assign to each bit.
        """
        ...

    @overload
    def __init__(self, bytes: typing.List[int]) -> None:
        """
        Initializes a new instance of the BitArray class that contains bit values copied
        from the specified array of bytes.
        
        :param bytes: An array of bytes containing the values to copy, where each byte represents eight consecutive bits.
        """
        ...

    @overload
    def __init__(self, values: typing.List[bool]) -> None:
        """
        Initializes a new instance of the BitArray class that contains bit values
        copied from the specified array of Booleans.
        
        :param values: An array of Booleans to copy.
        """
        ...

    @overload
    def __init__(self, values: typing.List[int]) -> None:
        """
        Initializes a new instance of the BitArray class that contains bit values
        copied from the specified array of 32-bit integers.
        
        :param values: An array of integers containing the values to copy, where each integer represents 32 consecutive bits.
        """
        ...

    @overload
    def __init__(self, bits: System.Collections.BitArray) -> None:
        """
        Initializes a new instance of the BitArray class that contains bit values copied from the specified BitArray.
        
        :param bits: The BitArray to copy.
        """
        ...

    def __setitem__(self, index: int, value: bool) -> None:
        """
        Gets or sets the value of the bit at a specific position in the BitArray.
        
        :param index: The zero-based index of the value to get or set.
        :returns: The value of the bit at position .
        """
        ...

    def And(self, value: System.Collections.BitArray) -> System.Collections.BitArray:
        """
        Performs the bitwise AND operation between the elements of the current BitArray object and the
        corresponding elements in the specified array. The current BitArray object will be modified to
        store the result of the bitwise AND operation.
        
        :param value: The array with which to perform the bitwise AND operation.
        :returns: An array containing the result of the bitwise AND operation, which is a reference to the current BitArray object.
        """
        ...

    def clone(self) -> System.Object:
        """Creates a shallow copy of the BitArray."""
        ...

    def copy_to(self, array: System.Array, index: int) -> None:
        ...

    def get(self, index: int) -> bool:
        """
        Gets the value of the bit at a specific position in the BitArray.
        
        :param index: The zero-based index of the value to get.
        :returns: The value of the bit at position .
        """
        ...

    def get_enumerator(self) -> System.Collections.IEnumerator:
        """
        Returns an enumerator that iterates through the BitArray.
        
        :returns: An IEnumerator for the entire BitArray.
        """
        ...

    def has_all_set(self) -> bool:
        """
        Determines whether all bits in the BitArray are set to true.
        
        :returns: true if every bit in the BitArray is set to true, or if BitArray is empty; otherwise, false.
        """
        ...

    def has_any_set(self) -> bool:
        """
        Determines whether any bit in the BitArray is set to true.
        
        :returns: true if BitArray is not empty and at least one of its bit is set to true; otherwise, false.
        """
        ...

    def left_shift(self, count: int) -> System.Collections.BitArray:
        """
        Shifts all the bit values of the current BitArray to the left on  bits.
        
        :param count: The number of shifts to make for each bit.
        :returns: The current BitArray.
        """
        ...

    def Not(self) -> System.Collections.BitArray:
        """
        Inverts all the bit values in the current BitArray, so that elements set to true are changed to false,
        and elements set to false are changed to true.
        
        :returns: The current instance with inverted bit values.
        """
        ...

    def Or(self, value: System.Collections.BitArray) -> System.Collections.BitArray:
        """
        Performs the bitwise OR operation between the elements of the current BitArray object and the
        corresponding elements in the specified array. The current BitArray object will be modified to
        store the result of the bitwise OR operation.
        
        :param value: The array with which to perform the bitwise OR operation.
        :returns: An array containing the result of the bitwise OR operation, which is a reference to the current BitArray object.
        """
        ...

    def right_shift(self, count: int) -> System.Collections.BitArray:
        """
        Shifts all the bit values of the current BitArray to the right on  bits.
        
        :param count: The number of shifts to make for each bit.
        :returns: The current BitArray.
        """
        ...

    def set(self, index: int, value: bool) -> None:
        """
        Sets the value of the bit at a specific position in the BitArray.
        
        :param index: The zero-based index of the value to get.
        :param value: The Boolean value to assign to the bit.
        """
        ...

    def set_all(self, value: bool) -> None:
        """
        Sets all bits in the BitArray to the specified value.
        
        :param value: The Boolean value to assign to all bits.
        """
        ...

    def xor(self, value: System.Collections.BitArray) -> System.Collections.BitArray:
        """
        Performs the bitwise XOR operation between the elements of the current BitArray object and the
        corresponding elements in the specified array. The current BitArray object will be modified to
        store the result of the bitwise XOR operation.
        
        :param value: The array with which to perform the bitwise XOR operation.
        :returns: An array containing the result of the bitwise XOR operation, which is a reference to the current BitArray object.
        """
        ...


class IStructuralComparable(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def compare_to(self, other: typing.Any, comparer: System.Collections.IComparer) -> int:
        ...


class ListDictionaryInternal(System.Object, System.Collections.IDictionary):
    """
    Implements IDictionary using a singly linked list.
    Recommended for collections that typically include fewer than 10 items.
    """

    @property
    def count(self) -> int:
        ...

    @property
    def keys(self) -> System.Collections.ICollection:
        ...

    @property
    def is_read_only(self) -> bool:
        ...

    @property
    def is_fixed_size(self) -> bool:
        ...

    @property
    def is_synchronized(self) -> bool:
        ...

    @property
    def sync_root(self) -> System.Object:
        ...

    @property
    def values(self) -> System.Collections.ICollection:
        ...

    def __getitem__(self, key: typing.Any) -> typing.Any:
        ...

    def __init__(self) -> None:
        ...

    def __setitem__(self, key: typing.Any, value: typing.Any) -> None:
        ...

    def add(self, key: typing.Any, value: typing.Any) -> None:
        ...

    def clear(self) -> None:
        ...

    def contains(self, key: typing.Any) -> bool:
        ...

    def copy_to(self, array: System.Array, index: int) -> None:
        ...

    def get_enumerator(self) -> System.Collections.IDictionaryEnumerator:
        ...

    def remove(self, key: typing.Any) -> None:
        ...


class IList(System.Collections.ICollection, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    @abc.abstractmethod
    def is_read_only(self) -> bool:
        ...

    @property
    @abc.abstractmethod
    def is_fixed_size(self) -> bool:
        ...

    def __getitem__(self, index: int) -> typing.Any:
        ...

    def __setitem__(self, index: int, value: typing.Any) -> None:
        ...

    def add(self, value: typing.Any) -> int:
        ...

    def clear(self) -> None:
        ...

    def contains(self, value: typing.Any) -> bool:
        ...

    def index_of(self, value: typing.Any) -> int:
        ...

    def insert(self, index: int, value: typing.Any) -> None:
        ...

    def remove(self, value: typing.Any) -> None:
        ...

    def remove_at(self, index: int) -> None:
        ...


class ArrayList(System.Object, System.Collections.IList, System.ICloneable):
    """Implements the IList interface using an array whose size is dynamically increased as required."""

    @property
    def capacity(self) -> int:
        ...

    @capacity.setter
    def capacity(self, value: int) -> None:
        ...

    @property
    def count(self) -> int:
        ...

    @property
    def is_fixed_size(self) -> bool:
        ...

    @property
    def is_read_only(self) -> bool:
        ...

    @property
    def is_synchronized(self) -> bool:
        ...

    @property
    def sync_root(self) -> System.Object:
        ...

    def __getitem__(self, index: int) -> typing.Any:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, capacity: int) -> None:
        ...

    @overload
    def __init__(self, c: System.Collections.ICollection) -> None:
        ...

    def __setitem__(self, index: int, value: typing.Any) -> None:
        ...

    @staticmethod
    def adapter(list: System.Collections.IList) -> System.Collections.ArrayList:
        ...

    def add(self, value: typing.Any) -> int:
        ...

    def add_range(self, c: System.Collections.ICollection) -> None:
        ...

    @overload
    def binary_search(self, index: int, count: int, value: typing.Any, comparer: System.Collections.IComparer) -> int:
        ...

    @overload
    def binary_search(self, value: typing.Any) -> int:
        ...

    @overload
    def binary_search(self, value: typing.Any, comparer: System.Collections.IComparer) -> int:
        ...

    def clear(self) -> None:
        ...

    def clone(self) -> System.Object:
        ...

    def contains(self, item: typing.Any) -> bool:
        ...

    @overload
    def copy_to(self, array: System.Array) -> None:
        ...

    @overload
    def copy_to(self, array: System.Array, array_index: int) -> None:
        ...

    @overload
    def copy_to(self, index: int, array: System.Array, array_index: int, count: int) -> None:
        ...

    @staticmethod
    @overload
    def fixed_size(list: System.Collections.IList) -> System.Collections.IList:
        ...

    @staticmethod
    @overload
    def fixed_size(list: System.Collections.ArrayList) -> System.Collections.ArrayList:
        ...

    @overload
    def get_enumerator(self) -> System.Collections.IEnumerator:
        ...

    @overload
    def get_enumerator(self, index: int, count: int) -> System.Collections.IEnumerator:
        ...

    def get_range(self, index: int, count: int) -> System.Collections.ArrayList:
        ...

    @overload
    def index_of(self, value: typing.Any) -> int:
        ...

    @overload
    def index_of(self, value: typing.Any, start_index: int) -> int:
        ...

    @overload
    def index_of(self, value: typing.Any, start_index: int, count: int) -> int:
        ...

    def insert(self, index: int, value: typing.Any) -> None:
        ...

    def insert_range(self, index: int, c: System.Collections.ICollection) -> None:
        ...

    @overload
    def last_index_of(self, value: typing.Any) -> int:
        ...

    @overload
    def last_index_of(self, value: typing.Any, start_index: int) -> int:
        ...

    @overload
    def last_index_of(self, value: typing.Any, start_index: int, count: int) -> int:
        ...

    @staticmethod
    @overload
    def read_only(list: System.Collections.IList) -> System.Collections.IList:
        ...

    @staticmethod
    @overload
    def read_only(list: System.Collections.ArrayList) -> System.Collections.ArrayList:
        ...

    def remove(self, obj: typing.Any) -> None:
        ...

    def remove_at(self, index: int) -> None:
        ...

    def remove_range(self, index: int, count: int) -> None:
        ...

    @staticmethod
    def repeat(value: typing.Any, count: int) -> System.Collections.ArrayList:
        ...

    @overload
    def reverse(self) -> None:
        ...

    @overload
    def reverse(self, index: int, count: int) -> None:
        ...

    def set_range(self, index: int, c: System.Collections.ICollection) -> None:
        ...

    @overload
    def sort(self) -> None:
        ...

    @overload
    def sort(self, comparer: System.Collections.IComparer) -> None:
        ...

    @overload
    def sort(self, index: int, count: int, comparer: System.Collections.IComparer) -> None:
        ...

    @staticmethod
    @overload
    def synchronized(list: System.Collections.IList) -> System.Collections.IList:
        ...

    @staticmethod
    @overload
    def synchronized(list: System.Collections.ArrayList) -> System.Collections.ArrayList:
        ...

    @overload
    def to_array(self) -> typing.List[System.Object]:
        ...

    @overload
    def to_array(self, type: typing.Type) -> System.Array:
        ...

    def trim_to_size(self) -> None:
        ...


class Comparer(System.Object, System.Collections.IComparer, System.Runtime.Serialization.ISerializable):
    """Compares two objects for equivalence, where string comparisons are case-sensitive."""

    DEFAULT: System.Collections.Comparer = ...

    DEFAULT_INVARIANT: System.Collections.Comparer = ...

    def __init__(self, culture: System.Globalization.CultureInfo) -> None:
        ...

    def compare(self, a: typing.Any, b: typing.Any) -> int:
        ...

    def get_object_data(self, info: System.Runtime.Serialization.SerializationInfo, context: System.Runtime.Serialization.StreamingContext) -> None:
        """Obsoletions.LegacyFormatterImplMessage"""
        warnings.warn("Obsoletions.LegacyFormatterImplMessage", DeprecationWarning)


class IStructuralEquatable(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def equals(self, other: typing.Any, comparer: System.Collections.IEqualityComparer) -> bool:
        ...

    def get_hash_code(self, comparer: System.Collections.IEqualityComparer) -> int:
        ...


class StructuralComparisons(System.Object):
    """This class has no documentation."""

    STRUCTURAL_COMPARER: System.Collections.IComparer

    STRUCTURAL_EQUALITY_COMPARER: System.Collections.IEqualityComparer





from typing import overload
from enum import Enum
import datetime
import typing

import QuantConnect
import QuantConnect.Data.Market
import QuantConnect.Securities
import QuantConnect.Securities.FutureOption
import QuantConnect.Securities.Option
import System


class FutureOptionSymbol(System.Object):
    """Static helper methods to resolve Futures Options Symbol-related tasks."""

    @staticmethod
    def get_last_day_of_trading(symbol: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract]) -> datetime.datetime:
        """
        Gets the last day of trading, aliased to be the Futures options' expiry
        
        :param symbol: Futures Options Symbol
        :returns: Last day of trading date.
        """
        ...

    @staticmethod
    def is_standard(_: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract]) -> bool:
        """
        Detects if the future option contract is standard, i.e. not weekly, not short-term, not mid-sized, etc.
        
        :param _: Symbol
        :returns: true.
        """
        ...


class FuturesOptionsExpiryFunctions(System.Object):
    """Futures options expiry lookup utility class"""

    @staticmethod
    def futures_option_expiry(canonical_future_option_symbol: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract], future_contract_month: typing.Union[datetime.datetime, datetime.date]) -> datetime.datetime:
        """
        Gets the Futures Options' expiry for the given contract month.
        
        :param canonical_future_option_symbol: Canonical Futures Options Symbol. Will be made canonical if not provided a canonical
        :param future_contract_month: Contract month of the underlying Future
        :returns: Expiry date/time.
        """
        ...

    @staticmethod
    def get_future_option_expiry_from_future_expiry(future_symbol: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract], canonical_future_option: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract] = None) -> datetime.datetime:
        """
        Gets the Future Option's expiry from the Future Symbol provided
        
        :param future_symbol: Future (non-canonical) Symbol
        :param canonical_future_option: The canonical Future Option Symbol
        :returns: Future Option Expiry for the Future with the same contract month.
        """
        ...


class FuturesOptionsUnderlyingMapper(System.Object):
    """Creates the underlying Symbol that corresponds to a futures options contract"""

    @staticmethod
    def get_future_contract_month_no_rules_applied(canonical_future_symbol: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract], future_option_expiration_date: typing.Union[datetime.datetime, datetime.date]) -> datetime.datetime:
        """
        Gets the theoretical (i.e. intermediate/naive) future contract month if we assumed a 1-1 mapping
        between FOPs contract months and Futures contract months, i.e. they share the same contract month.
        
        :param canonical_future_symbol: Canonical future Symbol
        :param future_option_expiration_date: Future Option Expiration Date
        :returns: Contract month assuming that the Future Option and Future share the same contract month.
        """
        ...

    @staticmethod
    def get_underlying_future_from_future_option(future_option_ticker: str, market: str, future_option_expiration: typing.Union[datetime.datetime, datetime.date], date: typing.Optional[datetime.datetime] = None) -> QuantConnect.Symbol:
        """
        Gets the FOP's underlying Future. The underlying Future's contract month might not match
        the contract month of the Future Option when providing CBOT or COMEX based FOPs contracts to this method.
        
        :param future_option_ticker: Future option ticker
        :param market: Market of the Future Option
        :param future_option_expiration: Expiration date of the future option
        :param date: Date to search the future chain provider with. Optional, but required for CBOT based contracts
        :returns: Symbol if there is an underlying for the FOP, null if there's no underlying found for the Future Option.
        """
        ...


class FutureOption(QuantConnect.Securities.Option.Option):
    """Futures Options security"""

    def __init__(self, symbol: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract], exchange_hours: QuantConnect.Securities.SecurityExchangeHours, quote_currency: QuantConnect.Securities.Cash, symbol_properties: QuantConnect.Securities.Option.OptionSymbolProperties, currency_converter: QuantConnect.Securities.ICurrencyConverter, registered_types: QuantConnect.Securities.IRegisteredSecurityDataTypesProvider, security_cache: QuantConnect.Securities.SecurityCache, underlying: QuantConnect.Securities.Security) -> None:
        """
        Constructor for the future option security
        
        :param symbol: Symbol of the future option
        :param exchange_hours: Exchange hours of the future option
        :param quote_currency: Quoted currency of the future option
        :param symbol_properties: Symbol properties of the future option
        :param currency_converter: Currency converter
        :param registered_types: Provides all data types registered to the algorithm
        :param security_cache: Cache of security objects
        :param underlying: Future underlying security
        """
        ...


class CMEStrikePriceScalingFactors(System.Object):
    """Provides a means to get the scaling factor for CME's quotes API"""

    @staticmethod
    def get_scale_factor(underlying_future: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract]) -> float:
        """
        Gets the option chain strike price scaling factor for the quote response from CME
        
        :param underlying_future: Underlying future Symbol to normalize
        :returns: Scaling factor for the strike price.
        """
        ...





from typing import overload
from enum import Enum
import typing

import System
import System.Buffers
import System.Text.Unicode


class Utf8(System.Object):
    """Provides static methods that convert chunked data between UTF-8 and UTF-16 encodings."""

    @staticmethod
    def from_utf_16(source: System.ReadOnlySpan[str], destination: System.Span[int], chars_read: typing.Optional[int], bytes_written: typing.Optional[int], replace_invalid_sequences: bool = True, is_final_block: bool = True) -> typing.Tuple[System.Buffers.OperationStatus, int, int]:
        """Transcodes the UTF-16  buffer to  as UTF-8."""
        ...

    @staticmethod
    def is_valid(value: System.ReadOnlySpan[int]) -> bool:
        """
        Validates that the value is well-formed UTF-8.
        
        :param value: The ReadOnlySpan{T} string.
        :returns: true if value is well-formed UTF-8, false otherwise.
        """
        ...

    @staticmethod
    def to_utf_16(source: System.ReadOnlySpan[int], destination: System.Span[str], bytes_read: typing.Optional[int], chars_written: typing.Optional[int], replace_invalid_sequences: bool = True, is_final_block: bool = True) -> typing.Tuple[System.Buffers.OperationStatus, int, int]:
        """Transcodes the UTF-8  buffer to  as UTF-16."""
        ...





from typing import overload
from enum import Enum
import abc
import typing

import System
import System.Collections
import System.Collections.Frozen
import System.Collections.Generic
import System.Collections.Immutable

System_Collections_Frozen_FrozenDictionary_TKey = typing.TypeVar("System_Collections_Frozen_FrozenDictionary_TKey")
System_Collections_Frozen_FrozenDictionary_TValue = typing.TypeVar("System_Collections_Frozen_FrozenDictionary_TValue")
System_Collections_Frozen_FrozenDictionary_AlternateLookup_TAlternateKey = typing.TypeVar("System_Collections_Frozen_FrozenDictionary_AlternateLookup_TAlternateKey")
System_Collections_Frozen_FrozenSet_T = typing.TypeVar("System_Collections_Frozen_FrozenSet_T")
System_Collections_Frozen_FrozenSet_AlternateLookup_TAlternate = typing.TypeVar("System_Collections_Frozen_FrozenSet_AlternateLookup_TAlternate")


class FrozenDictionary(typing.Generic[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue], System.Object, System.Collections.Generic.IDictionary[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue], System.Collections.Generic.IReadOnlyDictionary[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue], System.Collections.IDictionary, typing.Iterable[System.Collections.Generic.KeyValuePair[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue]], metaclass=abc.ABCMeta):
    """Provides a set of initialization methods for instances of the FrozenDictionary{TKey, TValue} class."""

    class AlternateLookup(typing.Generic[System_Collections_Frozen_FrozenDictionary_AlternateLookup_TAlternateKey]):
        """
        Provides a type that may be used to perform operations on a FrozenDictionary{TKey, TValue}
        using a TAlternateKey as a key instead of a TKey.
        """

        @property
        def dictionary(self) -> System.Collections.Frozen.FrozenDictionary[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue]:
            """Gets the FrozenDictionary{TKey, TValue} against which this instance performs operations."""
            ...

        def __getitem__(self, key: System_Collections_Frozen_FrozenDictionary_AlternateLookup_TAlternateKey) -> System_Collections_Frozen_FrozenDictionary_TValue:
            """
            Gets or sets the value associated with the specified alternate key.
            
            :param key: The alternate key of the value to get or set.
            """
            ...

        def contains_key(self, key: System_Collections_Frozen_FrozenDictionary_AlternateLookup_TAlternateKey) -> bool:
            """
            Determines whether the FrozenDictionary{TKey, TValue} contains the specified alternate key.
            
            :param key: The alternate key to check.
            :returns: true if the key is in the dictionary; otherwise, false.
            """
            ...

        def try_get_value(self, key: System_Collections_Frozen_FrozenDictionary_AlternateLookup_TAlternateKey, value: typing.Optional[System_Collections_Frozen_FrozenDictionary_TValue]) -> typing.Tuple[bool, System_Collections_Frozen_FrozenDictionary_TValue]:
            """
            Gets the value associated with the specified alternate key.
            
            :param key: The alternate key of the value to get.
            :param value: When this method returns, contains the value associated with the specified key, if the key is found; otherwise, the default value for the type of the value parameter.
            :returns: true if an entry was found; otherwise, false.
            """
            ...

    class Enumerator(System.Collections.Generic.IEnumerator[System.Collections.Generic.KeyValuePair[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue]]):
        """Enumerates the elements of a FrozenDictionary{TKey, TValue}."""

        @property
        def current(self) -> System.Collections.Generic.KeyValuePair[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue]:
            ...

        def move_next(self) -> bool:
            ...

    EMPTY: System.Collections.Frozen.FrozenDictionary[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue]
    """Gets an empty FrozenDictionary{TKey, TValue}."""

    @property
    def comparer(self) -> System.Collections.Generic.IEqualityComparer[System_Collections_Frozen_FrozenDictionary_TKey]:
        """Gets the comparer used by this dictionary."""
        ...

    @property
    def keys(self) -> System.Collections.Immutable.ImmutableArray[System_Collections_Frozen_FrozenDictionary_TKey]:
        """Gets a collection containing the keys in the dictionary."""
        ...

    @property
    def values(self) -> System.Collections.Immutable.ImmutableArray[System_Collections_Frozen_FrozenDictionary_TValue]:
        """Gets a collection containing the values in the dictionary."""
        ...

    @property
    def count(self) -> int:
        """Gets the number of key/value pairs contained in the dictionary."""
        ...

    def __contains__(self, key: System_Collections_Frozen_FrozenDictionary_TKey) -> bool:
        """
        Determines whether the dictionary contains the specified key.
        
        :param key: The key to locate in the dictionary.
        :returns: true if the dictionary contains an element with the specified key; otherwise, false.
        """
        ...

    def __getitem__(self, key: System_Collections_Frozen_FrozenDictionary_TKey) -> typing.Any:
        """
        Gets a reference to the value associated with the specified key.
        
        :param key: The key of the value to get.
        :returns: A reference to the value associated with the specified key.
        """
        ...

    def __iter__(self) -> typing.Iterator[System.Collections.Generic.KeyValuePair[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue]]:
        ...

    def __len__(self) -> int:
        ...

    def contains_key(self, key: System_Collections_Frozen_FrozenDictionary_TKey) -> bool:
        """
        Determines whether the dictionary contains the specified key.
        
        :param key: The key to locate in the dictionary.
        :returns: true if the dictionary contains an element with the specified key; otherwise, false.
        """
        ...

    @overload
    def copy_to(self, destination: typing.List[System.Collections.Generic.KeyValuePair[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue]], destination_index: int) -> None:
        """
        Copies the elements of the dictionary to an array of type KeyValuePair{TKey, TValue}, starting at the specified .
        
        :param destination: The array that is the destination of the elements copied from the dictionary.
        :param destination_index: The zero-based index in  at which copying begins.
        """
        ...

    @overload
    def copy_to(self, destination: System.Span[System.Collections.Generic.KeyValuePair[System_Collections_Frozen_FrozenDictionary_TKey, System_Collections_Frozen_FrozenDictionary_TValue]]) -> None:
        """
        Copies the elements of the dictionary to a span of type KeyValuePair{TKey, TValue}.
        
        :param destination: The span that is the destination of the elements copied from the dictionary.
        """
        ...

    def get_enumerator(self) -> System.Collections.Frozen.FrozenDictionary.Enumerator:
        """
        Returns an enumerator that iterates through the dictionary.
        
        :returns: An enumerator that iterates through the dictionary.
        """
        ...

    def get_value_ref_or_null_ref(self, key: System_Collections_Frozen_FrozenDictionary_TKey) -> typing.Any:
        """
        Gets either a reference to a TValue in the dictionary or a null reference if the key does not exist in the dictionary.
        
        :param key: The key used for lookup.
        :returns: A reference to a TValue in the dictionary or a null reference if the key does not exist in the dictionary.
        """
        ...

    def try_get_value(self, key: System_Collections_Frozen_FrozenDictionary_TKey, value: typing.Optional[System_Collections_Frozen_FrozenDictionary_TValue]) -> typing.Tuple[bool, System_Collections_Frozen_FrozenDictionary_TValue]:
        """
        Gets the value associated with the specified key.
        
        :param key: The key of the value to get.
        :param value: When this method returns, contains the value associated with the specified key, if the key is found; otherwise, the default value for the type of the value parameter.
        :returns: true if the dictionary contains an element with the specified key; otherwise, false.
        """
        ...


class FrozenSet(typing.Generic[System_Collections_Frozen_FrozenSet_T], System.Object, System.Collections.Generic.ISet[System_Collections_Frozen_FrozenSet_T], System.Collections.Generic.IReadOnlyCollection[System_Collections_Frozen_FrozenSet_T], System.Collections.ICollection, typing.Iterable[System_Collections_Frozen_FrozenSet_T], metaclass=abc.ABCMeta):
    """Provides an immutable, read-only set optimized for fast lookup and enumeration."""

    class Enumerator(System.Collections.Generic.IEnumerator[System_Collections_Frozen_FrozenSet_T]):
        """Enumerates the values of a FrozenSet{T}."""

        @property
        def current(self) -> System_Collections_Frozen_FrozenSet_T:
            ...

        def move_next(self) -> bool:
            ...

    class AlternateLookup(typing.Generic[System_Collections_Frozen_FrozenSet_AlternateLookup_TAlternate]):
        """
        Provides a type that may be used to perform operations on a FrozenSet{T}
        using a TAlternate as a key instead of a T.
        """

        @property
        def set(self) -> System.Collections.Frozen.FrozenSet[System_Collections_Frozen_FrozenSet_T]:
            """Gets the FrozenSet{T} against which this instance performs operations."""
            ...

        def contains(self, item: System_Collections_Frozen_FrozenSet_AlternateLookup_TAlternate) -> bool:
            """
            Determines whether a set contains the specified element.
            
            :param item: The element to locate in the set.
            :returns: true if the set contains the specified element; otherwise, false.
            """
            ...

        def try_get_value(self, equal_value: System_Collections_Frozen_FrozenSet_AlternateLookup_TAlternate, actual_value: typing.Optional[System_Collections_Frozen_FrozenSet_T]) -> typing.Tuple[bool, System_Collections_Frozen_FrozenSet_T]:
            """
            Searches the set for a given value and returns the equal value it finds, if any.
            
            :param equal_value: The value to search for.
            :param actual_value: The value from the set that the search found, or the default value of T when the search yielded no match.
            :returns: A value indicating whether the search was successful.
            """
            ...

    EMPTY: System.Collections.Frozen.FrozenSet[System_Collections_Frozen_FrozenSet_T]
    """Gets an empty FrozenSet{T}."""

    @property
    def comparer(self) -> System.Collections.Generic.IEqualityComparer[System_Collections_Frozen_FrozenSet_T]:
        """Gets the comparer used by this set."""
        ...

    @property
    def items(self) -> System.Collections.Immutable.ImmutableArray[System_Collections_Frozen_FrozenSet_T]:
        """Gets a collection containing the values in the set."""
        ...

    @property
    def count(self) -> int:
        """Gets the number of values contained in the set."""
        ...

    def __iter__(self) -> typing.Iterator[System_Collections_Frozen_FrozenSet_T]:
        ...

    def contains(self, item: System_Collections_Frozen_FrozenSet_T) -> bool:
        """
        Determines whether the set contains the specified element.
        
        :param item: The element to locate.
        :returns: true if the set contains the specified element; otherwise, false.
        """
        ...

    @overload
    def copy_to(self, destination: typing.List[System_Collections_Frozen_FrozenSet_T], destination_index: int) -> None:
        """
        Copies the values in the set to an array, starting at the specified .
        
        :param destination: The array that is the destination of the values copied from the set.
        :param destination_index: The zero-based index in  at which copying begins.
        """
        ...

    @overload
    def copy_to(self, destination: System.Span[System_Collections_Frozen_FrozenSet_T]) -> None:
        """
        Copies the values in the set to a span.
        
        :param destination: The span that is the destination of the values copied from the set.
        """
        ...

    def get_enumerator(self) -> System.Collections.Frozen.FrozenSet.Enumerator:
        """
        Returns an enumerator that iterates through the set.
        
        :returns: An enumerator that iterates through the set.
        """
        ...

    def is_proper_subset_of(self, other: System.Collections.Generic.IEnumerable[System_Collections_Frozen_FrozenSet_T]) -> bool:
        ...

    def is_proper_superset_of(self, other: System.Collections.Generic.IEnumerable[System_Collections_Frozen_FrozenSet_T]) -> bool:
        ...

    def is_subset_of(self, other: System.Collections.Generic.IEnumerable[System_Collections_Frozen_FrozenSet_T]) -> bool:
        ...

    def is_superset_of(self, other: System.Collections.Generic.IEnumerable[System_Collections_Frozen_FrozenSet_T]) -> bool:
        ...

    def overlaps(self, other: System.Collections.Generic.IEnumerable[System_Collections_Frozen_FrozenSet_T]) -> bool:
        ...

    def set_equals(self, other: System.Collections.Generic.IEnumerable[System_Collections_Frozen_FrozenSet_T]) -> bool:
        ...

    def try_get_value(self, equal_value: System_Collections_Frozen_FrozenSet_T, actual_value: typing.Optional[System_Collections_Frozen_FrozenSet_T]) -> typing.Tuple[bool, System_Collections_Frozen_FrozenSet_T]:
        """
        Searches the set for a given value and returns the equal value it finds, if any.
        
        :param equal_value: The value to search for.
        :param actual_value: The value from the set that the search found, or the default value of T when the search yielded no match.
        :returns: A value indicating whether the search was successful.
        """
        ...


